@@ -3,7 +3,7 @@ var searchData=
   ['allowdisconnectedsendatanytime_0',['allowDisconnectedSendAtAnyTime',['../struct_m_q_t_t_async__create_options.html#abe7fdbe18bfd3577a75d3b386d69406c',1,'MQTTAsync_createOptions']]],
   ['alt_1',['alt',['../struct_m_q_t_t_async__success_data.html#afbc1fee4467369fefa30cb07047fca14',1,'MQTTAsync_successData::alt()'],['../struct_m_q_t_t_async__success_data5.html#a4bde812772718b8051b0d6e2000a5f5c',1,'MQTTAsync_successData5::alt()']]],
   ['array_2',['array',['../struct_m_q_t_t_properties.html#a3ac4c38b423393c1553dcf8b71e7dd58',1,'MQTTProperties']]],
-  ['automatic_20reconnect_3',['Automatic Reconnect',['../auto_reconnect.html',1,'']]],
-  ['automaticreconnect_4',['automaticReconnect',['../struct_m_q_t_t_async__connect_options.html#a7902ce4d11b96d8b19582bdd1f82b630',1,'MQTTAsync_connectOptions']]],
-  ['asynchronous_20mqtt_20client_20library_20for_20c_20_28mqttasync_29_5',['Asynchronous MQTT client library for C (MQTTAsync)',['../index.html',1,'']]]
+  ['asynchronous_20mqtt_20client_20library_20for_20c_20_28mqttasync_29_3',['Asynchronous MQTT client library for C (MQTTAsync)',['../index.html',1,'']]],
+  ['automatic_20reconnect_4',['Automatic Reconnect',['../auto_reconnect.html',1,'']]],
+  ['automaticreconnect_5',['automaticReconnect',['../struct_m_q_t_t_async__connect_options.html#a7902ce4d11b96d8b19582bdd1f82b630',1,'MQTTAsync_connectOptions']]]
 ];
